/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query;

import java.util.Collection;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Criterion;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Order;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Restrictions;


/**
 * <tt>Criteria</tt> is a simplified API for retrieving JCR Nodes by composing <tt>Criterion</tt> objects. This is a
 * very convenient approach for functionality like "search" screens where there is a variable number of conditions to be
 * placed upon the result set.<br>
 * The <tt>JCRCriteriaFactory</tt> is a factory for <tt>Criteria</tt>. <tt>Criterion</tt> instances are usually obtained
 * via the factory methods on <tt>Restrictions</tt>. eg.
 * 
 * <pre>
 * Calendar begin = Calendar.getInstance();
 * begin.set(2004, Calendar.JANUARY, 1);
 * Calendar end = Calendar.getInstance();
 * end.set(2008, Calendar.DECEMBER, 1);
 *   	
 * Collection pets = JCRCriteriaFactory.createMgnlCriteria(&quot;//dogs//*&quot;, MgnlContext.getQueryManager(&quot;website&quot;), &quot;nt:base&quot;).add(
 * 		Restrictions.contains(&quot;@name&quot;, &quot;Nana&quot;)).add(
 * 		Restrictions.gt(&quot;@weight&quot;, Float.valueOf(10))).add(
 * 		Restrictions.between(&quot;@birthDate&quot;, begin, end).addOrder(
 * 		Order.desc(&quot;@name&quot;)).list();
 * </pre>
 * 
 * will be translated into the following xpath statement
 * 
 * <pre>
 *  //dogs//*[((jcr:contains(@name, 'Nana')) and (@weight&gt;10.0) and (@birthDate &gt;=xs:dateTime('2004-01-01T00:00:00.000+00:00') and @birthDate &lt;=xs:dateTime('2008-12-01T23:59:59.000+00:00')))] order by @name descending
 * </pre>
 * 
 * You can also specify a different type to be returned in the Collection of results. eg.
 * 
 * <pre>
 * Collection&lt;Pet&gt; pets = JCRCriteriaFactory.createMgnlCriteria("//dogs//*", MgnlContext.getQueryManager("website"), "mgnl:content", Pet.class).add(
 *      Restrictions.contains("@name", "Nana")).add(
 *      Restrictions.gt("@weight", Float.valueOf(10))).add(
 *      Restrictions.between("@birthDate", begin, end).addOrder(
 *      Order.desc("@name")).list();
 * </pre>
 * 
 * Internally, this will use info.magnolia.content2bean.Content2BeanUtil.toBean() to transform nodes into beans. So, for
 * example, if you have a domain Pet class like this
 * 
 * <pre>
 * public class Pet
 * {
 *  private String name;
 *  private Float weight;
 *  private Calendar birthDate;
 *  //getters and setters here...
 * }
 * </pre>
 * 
 * Content nodes returned by the above query will be automatically converted to and populate instances of the
 * <code>Pet</code> type. Furthermore, you may want to have only a subset of the whole result set returned, much like in
 * a MySQL limit clause. In this case, you will use the <code>JCRCriteriaFactory.createMgnlCriteriaWithLimit</code> factory method.
 * For this to work, the underlying JCR repository implementation must support this feature (Jackrabbit 1.4+ does). Here
 * is an example.
 * 
 *<pre>
 * Collection&lt;Pet&gt; pets = JCRCriteriaFactory.createMgnlCriteriaWithLimit("//dogs//*", MgnlContext.getQueryManager("website"), "mgnl:content", Pet.class).add(
 *      Restrictions.contains("@name", "Nana")).add(
 *      Restrictions.gt("@weight", Float.valueOf(10))).add(
 *      Restrictions.between("@birthDate", begin, end).
 *      setFirstResult(5).
 *      setMaxResults(10).
 *      addOrder(Order.desc("@name")).list();
 *</pre>
 * 
 * Notice the <code>setFirstResult(int)</code> and <code>setMaxResults(int)</code> methods. Now calling
 * <code>list()</code> will return a subset of only five <code>Pet</code> objects, starting from the 6th item (counting
 * starts from 0). If you dont specify these two calls, <code>list()</code> will behave as usual by returning the entire result set.
 * If you only call <code>setMaxResults(int)</code>, the result set will be the subset of elements <code>[0, maxResults]</code>
 * (firstResultValue is 0 by default). 
 * <br><br>
 * A word of warning about implementations returned by <code>JCRCriteriaFactory</code>. They are <strong>NOT</strong> thread-safe,
 * therefore client code wishing to use one of them as a shared global variable <strong>MUST</strong> coordinate access
 * to it. These objects are actually meant to be instantiated and used within a method scope (e.g. a service method),
 * where no concurrent issues arise. <br>
 * <br>
 * This API was inspired by Hibernate's Criteria API. <br><br>
 * @see JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String)
 * @see JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String, Class)
 * @see JCRCriteriaFactory#createMgnlCriteriaWithLimit(String, info.magnolia.cms.core.search.QueryManager, String)
 * @see JCRCriteriaFactory#createMgnlCriteriaWithLimit(String, info.magnolia.cms.core.search.QueryManager, String,
 * Class)
 * @see Restrictions
 * @see Order
 * @author Federico Grilli
 * @version $Id: Criteria.java 1319 2009-08-28 11:23:30Z fgrilli $
 */
public interface Criteria
{

    /**
     * Add a {@link Criterion restriction} to constrain the results to be retrieved.
     * @param criterion The {@link Criterion criterion} object representing the restriction to be applied.
     * @return this (for method chaining)
     */
    public Criteria add(Criterion criterion);

    /**
     * Add an {@link Order ordering} to the result set. Only <strong>one Order criterion per query</strong> can be
     * applied. Any Order added after the first one will be ignored.
     * @param order The {@link Order order} object representing an ordering to be applied to the results.
     * @return this (for method chaining)
     */
    public Criteria addOrder(Order order);

    /**
     * Set a limit upon the number of objects to be retrieved.
     * @param maxResults the maximum number of results
     * @return this (for method chaining)
     */
    public Criteria setMaxResults(int maxResults);

    /**
     * Set the first result to be retrieved.
     * @param firstResult the first result to retrieve, numbered from <tt>0</tt>
     * @return this (for method chaining)
     */
    public Criteria setFirstResult(int firstResult);

    /*
     * Set a timeout for the underlying query.
     * @param timeout The timeout value to apply.
     * @return this (for method chaining) FIXME is it possible in JCR? public Criteria setTimeout(int timeout);
     */

    /**
     * Get the results. <strong>The implementation should guarantee the caller that the returned Collection is never
     * null</strong>
     * @return The Collection of matched query results.
     */
    public Collection< ? > list() throws JCRQueryException;
}