/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.utils;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * A utility class to escape xpath strings
 * @author fgrilli
 * @version $Id: XPathTextUtils.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public class XPathTextUtils
{

    private static final Logger log = LoggerFactory.getLogger(XPathTextUtils.class);

    private XPathTextUtils()
    {
    }

    /**
     * Escapes all illegal XPath search characters of a string.
     * <p>
     * Example:<br>
     * A search string like 'test?' will run into a ParseException documented in
     * http://issues.apache.org/jira/browse/JCR-1248 This is copied from org.apache.jackrabbit.util.Text and was put
     * here just to keep this API JCR implementation agnostic
     * @param string the string to encode
     * @return the escaped string
     */
    public static String escapeIllegalXpathSearchChars(String s)
    {
        StringBuffer sb = new StringBuffer();
        sb.append(s.substring(0, (s.length() - 1)));
        char c = s.charAt(s.length() - 1);
        if (c == '!'
            || c == '('
            || c == ':'
            || c == '^'
            || c == '['
            || c == ']'
            || c == '\"'
            || c == '{'
            || c == '}'
            || c == '?')
        {
            sb.append('\\');
        }
        sb.append(c);
        return sb.toString();
    }

    /**
     * Convert a string to an XPath 2.0 string literal, suitable for inclusion in a query. See JSR-170 spec v1.0, Sec.
     * 6.6.4.9.
     * @param str Any string.
     * @return A valid XPath 2.0 string literal, including enclosing quotes.
     */
    public static String stringToXPathLiteral(String str)
    {
        // Single quotes needed for jcr:contains()
        return str.replaceAll("'", "\"");
    }

    /**
     * Convert a string to a JCR search expression literal, suitable for use in jcr:contains() (inside XPath queries).
     * The characters - and " have special meaning, and may be escaped with a backslash to obtain their literal value.
     * See JSR-170 spec v1.0, Sec. 6.6.5.2.
     * @param str Any string.
     * @return A valid XPath 2.0 string literal suitable for use in jcr:contains(), including enclosing quotes.
     */
    public static String stringToJCRSearchExp(String str)
    {
        // Escape ' and \ everywhere, preceding them with \ except when \
        // appears
        // in one of the combinations \" or \-
        return escapeIllegalXpathSearchChars(stringToXPathLiteral(str
            .replaceAll("\\\\(?![-\"])", "\\\\\\\\")
            .replaceAll("'", "\\\\'")));
    }

    /**
     * @param String path to encode eg //my//path/2009//*
     * @return String encoded path eg //my//path/_x32_009//*
     */
    public static String encodeDigitsInPath(String path)
    {
        log.debug("path to encode is {}", path);
        if (StringUtils.isBlank(path))
        {
            String msg = "path cannot be a null or empty string";
            log.error(msg);
            throw new IllegalArgumentException(msg);
        }

        StringBuilder encodedPath = new StringBuilder(path.length());
        // TODO maybe a more robust check is needed
        for (int i = 0; i < path.length(); ++i)
        {
            char ch = path.charAt(i);

            if (i > 0 && path.charAt(i - 1) == '/' && Character.isDigit(ch))
            {
                encodedPath.append("_x" + Integer.toHexString(ch) + "_");
            }
            else
            {
                encodedPath.append(ch);
            }
        }
        log.debug("returning encoded path {}", encodedPath);
        return encodedPath.toString();
    }
}